program PngResize;
{************************************************************************
 * This program is  free software; you can redistribute it  and modify it
 * under the terms of the GNU  General Public License as published by the
 * Free Software Foundation; either version 2 of the license  or (at your
 * option) any later version.
 ***********************************************************************}

{$APPTYPE CONSOLE}
{$R *.res}

uses
  Windows,
  SysUtils,
  Graphics,
  pngimage,
  uCommon in 'uCommon.pas',
  uVerbose in 'uVerbose.pas';

var
   png:TPNGImage;
   offset, new_offset :TPoint;
   has_offset, offx_odd, offy_odd, pngx_odd, pngy_odd: boolean;
   save_as : string;

{ MAIN CODE }
begin
    if ParamCount() < 2 then
       begin
       SetConsoleTitle(PChar('PNG Resize v'+APPVER+' usage information'));
       // show usage
       ShowUsage();
       exit;
       end;

   if (LowerCase(ParamStr(1)) <> 'z1') AND (LowerCase(ParamStr(1)) <> 'z2') then
      begin
        ConsoleColor(true,true,false,false);
        writeln('ERROR! Unsupported sprite zoom level requested!');
        ConsoleColor(true,true,true,false);
        writeln('Please see USAGE for valid parameters');
        ConsoleColor();
        writeln('Exiting...');
        exit;
      end;




  try
    png := TPNGImage.Create;
    png.LoadFromFile(ParamStr(2));

{
If the size of the sprite and the offset are even, the sprite could be scaled down without problems.
If the size is odd and the offset is even, the sprite needs a new line at the end (right or bottom border).
If the size and the offset are odd, the sprite needs a new line at the start (top or left border).
If the size of the sprite is even and the offset is odd, the sprite needs two new lines, at the start and the sides.
}
    // OFFSETS
    offx_odd := true;
    offy_odd := true;
    offset := GetPNGOffset(png, has_offset);
    if has_offset then
       begin
       if (offset.X mod 2) = 0 then offx_odd := false;
       if (offset.Y mod 2) = 0 then offy_odd := false;
       end
    else
       begin
       ShowWarning('Sprite has no offsets defined. Assuming even offsets!');
       end;
    // IMAGE SIZE
    pngx_odd := true;
    pngy_odd := true;
    if (png.Header.Width  mod 2) = 0 then pngx_odd := false;
    if (png.Header.Height mod 2) = 0 then pngy_odd := false;

    if not pngx_odd AND not pngy_odd then ShowInfo('Sprite has even sizes!', true)
                                     else ShowInfo('Sprite has odd sizes!');
    if not offx_odd AND not offy_odd then ShowInfo('Offsets are even!', true)
                                     else ShowInfo('Offsets are odd!');

// adjust sprite size X
    // size odd, offset even. add line to end
    if pngx_odd AND not offx_odd then
       begin
       AddPNG_Xline(png);
       end
    else
    // size odd, offset odd, add line at start
    if pngx_odd AND offx_odd then
       begin
       AddPNG_Xline(png, false);
       offset.X := offset.X - 1; // correct offset position
       end
    else
    // size even, offset odd, add 2 lines (both directions)
    if not pngx_odd AND offx_odd then
       begin
       AddPNG_Xline(png, false,true);
       offset.X := offset.X - 1; // correct offset position
       end;

// adjust sprite size Y
    // size odd, offset even. add line to end
    if pngy_odd AND not offy_odd then
       begin
       AddPNG_Yline(png);
       end
    else
    // size odd, offset odd, add line at start
    if pngy_odd AND offy_odd then
       begin
       AddPNG_Yline(png, false);
       offset.Y := offset.Y - 1; // correct offset position
       end
    else
    // size even, offset odd, add 2 lines (both directions)
    if not pngy_odd AND offy_odd then
       begin
       AddPNG_Yline(png, false,true);
       offset.Y := offset.Y - 1; // correct offset position
       end;

    // resize sprite
    if LowerCase(ParamStr(1))='z1' then
       begin
       PNG_Div(png,2);             // redundant but won't do anything if not needed
       SmoothResize(png, (png.Header.Width div 2), (png.Header.Height div 2));
       new_offset.X := offset.X div 2;
       new_offset.Y := offset.Y div 2;
       end
    else
       begin
       PNG_Div(png,4);             // make sure its dividable by 4 (mod 4 = 0)
       SmoothResize(png, (png.Header.Width div 4), (png.Header.Height div 4));
       new_offset.X := offset.X div 4;
       new_offset.Y := offset.Y div 4;
       end;

    // save resized sprite
    if ParamCount() >= 3 then
       save_as := ParamStr(3)
    else
       begin
       if Pos('_z0',LowerCase(ParamStr(2)))>0 then
          save_as := StringReplace(LowerCase(ParamStr(2)),'_z0','_'+LowerCase(ParamStr(1)),[rfReplaceAll])
       else
          save_as := ChangeFileExt(ParamStr(2),'')+'_'+LowerCase(ParamStr(1))+'.png';
       end;

    SaveSprite(png,save_as,new_offset,has_offset);
    ConsoleColor(true,true,true,false);
    Write('New sprite saved as: ');
    ConsoleCOlor(true,false,true,false);
    Writeln(save_as);
    ConsoleColor(true,false,true,true);
    Writeln('Offset information:');
    if not has_offset then
       begin
       ConsoleColor(true,true,false,false);
       writeln('No offsets in original sprite. No offsets saved!');
       end
    else
       begin
       ConsoleColor(true,true,true,false);
       Writeln(#9+'Original offsets'+#9+'New offsets');
       ConsoleColor(true);
       Write('Left:'+#9#9+IntToStr(offset.X));
       ConsoleColor(true,false,true);
       Writeln(#9#9+'     '+IntToStr(new_offset.X));
       ConsoleColor(true);
       Write('Top :'+#9#9+IntToStr(offset.X));
       ConsoleColor(true,false,true);
       Writeln(#9#9+'     '+IntToStr(new_offset.X));
       end;

    ConsoleColor();
    Writeln(' '); // reset console color


  except
    on E:Exception do
      Writeln(E.Classname, ': ', E.Message);
  end;
end.
