unit uCommon;
{************************************************************************
 * This program is  free software; you can redistribute it  and modify it
 * under the terms of the GNU  General Public License as published by the
 * Free Software Foundation; either version 2 of the license  or (at your
 * option) any later version.
 ***********************************************************************}

interface

uses
  Windows, Classes, SysUtils, pngimage, uVerbose;

procedure SmoothResize(apng:TPngImage; NuWidth,NuHeight:integer);
function GetPNGOffset(png: TPngImage; var has_offset:boolean):TPoint;
procedure SaveSprite(sprite:TPngImage; path:string; offset:TPoint; saveoffset:boolean=true);

procedure PNG_Div(png:TPngImage; ammount:cardinal=4);
procedure AddPNG_Xline(png:TPngImage; right:boolean=true; both:boolean=false);
procedure AddPNG_Yline(png:TPngImage; bottom:boolean=true; both:boolean=false);

implementation

procedure SmoothResize(apng:TPngImage; NuWidth,NuHeight:integer);
var
  xscale, yscale         : Single;
  sfrom_y, sfrom_x       : Single;
  ifrom_y, ifrom_x       : Integer;
  to_y, to_x             : Integer;
  weight_x, weight_y     : array[0..1] of Single;
  weight                 : Single;
  new_red, new_green     : Integer;
  new_blue, new_alpha    : Integer;
  new_colortype          : Integer;
  total_red, total_green : Single;
  total_blue, total_alpha: Single;
  IsAlpha                : Boolean;
  ix, iy                 : Integer;
  bTmp : TPNGImage;
  sli, slo : pRGBLine;
  ali, alo: pbytearray;
begin
  if not (apng.Header.ColorType in [COLOR_RGBALPHA, COLOR_RGB]) then
    raise Exception.Create('Only COLOR_RGBALPHA and COLOR_RGB formats' +
    ' are supported');
  IsAlpha := apng.Header.ColorType in [COLOR_RGBALPHA];
  if IsAlpha then new_colortype := COLOR_RGBALPHA else
    new_colortype := COLOR_RGB;
  bTmp := TPNGImage.CreateBlank(new_colortype, 8, NuWidth, NuHeight);
  xscale := bTmp.Width / (apng.Width-1);
  yscale := bTmp.Height / (apng.Height-1);
  for to_y := 0 to bTmp.Height-1 do begin
    sfrom_y := to_y / yscale;
    ifrom_y := Trunc(sfrom_y);
    weight_y[1] := sfrom_y - ifrom_y;
    weight_y[0] := 1 - weight_y[1];
    for to_x := 0 to bTmp.Width-1 do begin
      sfrom_x := to_x / xscale;
      ifrom_x := Trunc(sfrom_x);
      weight_x[1] := sfrom_x - ifrom_x;
      weight_x[0] := 1 - weight_x[1];

      total_red   := 0.0;
      total_green := 0.0;
      total_blue  := 0.0;
      total_alpha  := 0.0;
      for ix := 0 to 1 do begin
        for iy := 0 to 1 do begin
          sli := apng.Scanline[ifrom_y + iy];
          if IsAlpha then ali := apng.AlphaScanline[ifrom_y + iy];
          new_red := sli[ifrom_x + ix].rgbtRed;
          new_green := sli[ifrom_x + ix].rgbtGreen;
          new_blue := sli[ifrom_x + ix].rgbtBlue;
          if IsAlpha then new_alpha := ali[ifrom_x + ix];
          weight := weight_x[ix] * weight_y[iy];
          total_red   := total_red   + new_red   * weight;
          total_green := total_green + new_green * weight;
          total_blue  := total_blue  + new_blue  * weight;
          if IsAlpha then total_alpha  := total_alpha  + new_alpha  * weight;
        end;
      end;
      slo := bTmp.ScanLine[to_y];
      if IsAlpha then alo := bTmp.AlphaScanLine[to_y];
      slo[to_x].rgbtRed := Round(total_red);
      slo[to_x].rgbtGreen := Round(total_green);
      slo[to_x].rgbtBlue := Round(total_blue);
      if isAlpha then alo[to_x] := Round(total_alpha);
    end;
  end;
  apng.Assign(bTmp);
  bTmp.Free;
end;



// Get PNG offsets from the PNG file if they already are defined, else sets them to zero
function GetPNGOffset(png: TPngImage; var has_offset:boolean):TPoint;
var p:TPoint;
    i:integer;
    tEXt: TChunktEXt;
    found_x, found_y:boolean;
begin
p.X:=0;
p.Y:=0;
found_x := false;
found_y := false;
try
begin
for i := (png.Chunks.Count - 1) downto 0 do
  begin
  if png.Chunks.Item[i].ClassType = TChunktEXt then
    begin
    tEXt := png.Chunks.Item[i] as TChunktEXt;
    if AnsiString(tEXt.Keyword)=AnsiString('x_offs') then
       begin
         p.X := StrToInt(trim(string(tEXt.Text)));
         found_x := true;
         // remove the chunk from the memory copy
         png.Chunks.RemoveChunk(tEXt);
       end
    else
    if AnsiString(tEXt.Keyword)=AnsiString('y_offs') then
       begin
         p.Y := StrToInt(trim(string(tEXt.Text)));
         found_y := true;
         // remove the chunk from the memory copy
         png.Chunks.RemoveChunk(tEXt);
       end;
    end
  end;
end
except
  // Something went wrong with the detection
  p.X := 0;  found_x := false;
  p.Y := 0;  found_y := false;
end;

if (not found_x) or (not found_y) then has_offset := false
                                  else has_offset := true;

Result:=p;
end;


procedure SaveSprite(sprite:TPngImage; path:string; offset:TPoint; saveoffset:boolean=true);
{Characters for the PNG header}
const
  PngHeader: Array[0..7] of AnsiChar = (#137, #80, #78, #71, #13, #10, #26, #10);

var j:integer;
    saveto:TFileStream;
    p:TPoint;
    tmp:boolean;
begin

if saveoffset then
   begin
   // clear existing x_offs and y_offs is present
   p:=GetPNGOffset(sprite, tmp);

   {
   compare original offsets and if they are the same,
   do nothing and cancel further sprite processing
   }
   //if (p.X = sprite.SpriteOffsetX) and (p.Y = sprite.SpriteOffsetY) then exit;

   // add new offsets defined
   sprite.AddtEXt('x_offs',AnsiString(IntToStr(offset.X)));
   sprite.AddtEXt('y_offs',AnsiString(IntToStr(offset.Y)));
   end;

// initialize stream
saveto := TFileStream.Create(path, fmCreate);
{Reads the header}
saveto.Write(PNGHeader[0], 8);
// write TChunkIHDR chunk
for j := 0 to sprite.Chunks.Count - 1 do
  if sprite.Chunks.Item[j] is TChunkIHDR then
     sprite.Chunks.Item[j].SaveToStream(saveto);
// write TChunktEXt chunks
for j := 0 to sprite.Chunks.Count - 1 do
  if sprite.Chunks.Item[j] is TChunktEXt then
     sprite.Chunks.Item[j].SaveToStream(saveto);

// write all the other chunks
for j := 0 to sprite.Chunks.Count - 1 do
  if (not (sprite.Chunks.Item[j] is TChunkIHDR)) and (not (sprite.Chunks.Item[j] is TChunktEXt)) then
     sprite.Chunks.Item[j].SaveToStream(saveto);

saveto.Free;
end;





procedure PNG_Div(png:TPngImage; ammount:cardinal=4);
var png2:TPngImage;
    x, y, x2, y2, a:cardinal;
    t,t2:Trect;
begin
x2 := png.Header.Width;
y2 := png.Header.Height;

a := x2 mod ammount;
if a <> 0 then
   begin
   x2 := x2+a;
   ShowInfo(#9+'Width extended by '+IntToStr(a)+' pixel(s)');
   end;

a:= y2 mod ammount;
if a <> 0 then
   begin
   y2 := y2+a;
   ShowInfo(#9+'Height extended by '+IntToStr(a)+' pixel(s)');
   end;

if (x2 > png.Header.width) or (y2 > png.Header.Height) then
   begin
   png2 := TPngImage.CreateBlank(COLOR_RGB, 8, x2, y2);
   png2.CreateAlpha;

   // set initial transparency
   for y := 0 to y2-1 do
     for x := 0 to x2 - 1 do
       png2.AlphaScanline[y]^[x] := 0;

   for y := 0 to png.Header.Height - 1 do
     for x := 0 to png.Header.Width - 1 do
       png2.AlphaScanline[y]^[x] := png.AlphaScanline[y]^[x];
   
   // copy gamma information
   png2.GammaTable := png.GammaTable;


   // copy image data
   t := Rect(0,0,png.Header.Width-1, png.Header.Height-1);
   t2 := rect(0,0,png.Header.Width-1, png.Header.Height-1);

   png2.canvas.CopyRect(t2,png.Canvas,t);

   png.Assign(png2);
   png2.Free;
   end;
end;




procedure AddPNG_Xline(png:TPngImage; right:boolean=true; both:boolean=false);
var png2:TPngImage;
    x, y, x2, y2, a:cardinal;
    t,t2:Trect;
begin
if both then right := false; // if both, right must be false so we enforce it
if both then a := 2 else a := 1;

x2 := png.Header.Width+a;
y2 := png.Header.Height;
png2 := TPngImage.CreateBlank(COLOR_RGB, 8, x2, y2);
png2.CreateAlpha;

// set initial transparency
for y := 0 to y2-1 do
  for x := 0 to x2 - 1 do
    png2.AlphaScanline[y]^[x] := 0;


// set transparency from source
if (right) then
   begin
   for y := 0 to png.Header.Height - 1 do
     for x := 0 to png.Header.Width - 1 do
      png2.AlphaScanline[y]^[x] := png.AlphaScanline[y]^[x];
   end
else
   begin
   for y := 0 to png.Header.Height - 1 do
     for x := 0 to png.Header.Width - 1 do
       png2.AlphaScanline[y]^[x+1] := png.AlphaScanline[y]^[x];
   end;

// copy gamma information
png2.GammaTable := png.GammaTable;

// copy image data
t := Rect(0,0,png.Header.Width-1, png.Header.Height-1);

if right then
   t2 := rect(0,0,png.Header.Width-1, png.Header.Height-1)
else
   t2 := rect(1,0,png.Header.Width-1, png.Header.Height-1);

png2.canvas.CopyRect(t2,png.Canvas,t);

png.Assign(png2);
png2.Free;
end;



procedure AddPNG_Yline(png:TPngImage; bottom:boolean=true; both:boolean=false);
var png2:TPngImage;
    x, y, x2, y2, a:cardinal;
    t,t2:Trect;
begin
if both then bottom := false; // if both, bottom must be false so we enforce it
if both then a := 2 else a := 1;


x2 := png.Header.Width;
y2 := png.Header.Height+a;
png2 := TPngImage.CreateBlank(COLOR_RGB, 8, x2, y2);
png2.CreateAlpha;

// set initial transparency
for y := 0 to y2-1 do
  for x := 0 to x2 - 1 do
    png2.AlphaScanline[y]^[x] := 0;


// set transparency from source
if (bottom) then
   begin
   for y := 0 to png.Header.Height - 1 do
     for x := 0 to png.Header.Width - 1 do
      png2.AlphaScanline[y]^[x] := png.AlphaScanline[y]^[x];
   end
else
   begin
   for y := 0 to png.Header.Height - 1 do
     for x := 0 to png.Header.Width - 1 do
       png2.AlphaScanline[y+1]^[x] := png.AlphaScanline[y]^[x];
   end;

// copy gamma information
png2.GammaTable := png.GammaTable;

// copy image data
t := Rect(0,0,png.Header.Width-1, png.Header.Height-1);

if bottom then
   t2 := rect(0,0,png.Header.Width-1, png.Header.Height-1)
else
   t2 := rect(0,1,png.Header.Width-1, png.Header.Height-1);

png2.canvas.CopyRect(t2,png.Canvas,t);

png.Assign(png2);
png2.Free;
end;


end.
